//
// Copyright 2000 Perforce Software.  All rights reserved.
//
// This file is part of Perforce - the FAST SCM System.
//
// p4wPasswdTestPane:
//   Used to test for password error

#ifndef p4wPasswdTestPane_H
#define p4wPasswdTestPane_H

#include "p4wPane.h"


class StrDict;


class p4wPasswdTestPane : public p4wPane {

public:

	p4wPasswdTestPane( p4wView & ParentView, p4wRequest & Request );
	virtual	~p4wPasswdTestPane();

	//
	// Render functions.
	
	virtual void Render( StrDict * varList );
	virtual void RenderError( char *data, int escapeHTML );
	virtual void RenderInfo( char *data, char level );
	virtual void Prompt( const StrPtr &msg, StrBuf &rsp, 
				int noEcho, Error *e );

	//
	// We actually care about the output of the groups command

	void SetCheckGroup(int b) { fCheckGroup = b; fHadGroupError = 1; };
	
	//
	// Returns true if we didnot find user in fRequest::fGroup
	
	int HadGroupError() { return fHadGroupError; };
	
	//
	// Returns true if we received an invalid password error
	
	int HadPasswordError() { return fHadPasswordError; };
	
	//
	// Returns true if we received a p4 login error
	
	int HadLoginError() { return fHadLoginError; };
	
	//
	// Returns true if we received a unicode client required
	// error
	
	int HadUnicodeError() { return fHadUnicodeError; };

	//
	// Returns true if we received a SSL error
	
	int HadSSLError() { return fHadSSLError; };

	//
	// Returns true if we client is unknown
	
	int IsUnknownClient() { return fUnknownClient; };
	
	//
	// Returns true if we encountered ANY error
	
	int GotError() { return fSetError; }
	
	//
	// Error text
	
	const StrPtr &GetErrorMsg() { return fErrorText; }
	
	//
	// Client Root
	
	const StrPtr &GetClientRoot() { return fClientRoot; }

	//
	// Signal to test if client is valid
	void SetTestClient(int b) { fTestClient = b; }
	
private:
	int			fCheckGroup;
	int			fHadGroupError;
	int			fHadPasswordError;
	int			fHadLoginError;
	int			fHadUnicodeError;
	int			fHadSSLError;
    int			fUnknownClient;
	int			fSetError;
	
	StrBuf		fErrorText;
	StrBuf		fClientRoot;
	
	int			fTestClient;
};

#endif // p4wPasswdTestPane_H
